@@ -34,7 +34,7 @@
 #include "mail/em-account-editor.h"
 #include "mail/em-config.h"
 #include "libedataserver/e-account.h"
-#include "e-util/e-error.h"
+#include "e-util/e-alert-dialog.h"
 
 #include "exchange-operations.h"
 #include "exchange-folder-size-display.h"
@@ -126,8 +126,8 @@ e_exchange_calendar_pcalendar_on_change (GtkTreeView *treeview, ESource *source)
 	if (!account)
 		return;
 
-	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
-	gtk_tree_selection_get_selected(selection, &model, &iter);
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
+	gtk_tree_selection_get_selected (selection, &model, &iter);
 
 	gtk_tree_model_get (model, &iter, CALENDARRURI_COL, &ruri, -1);
 	es_ruri = g_strconcat (account->account_filename, "/;", ruri, NULL);
@@ -185,7 +185,7 @@ e_exchange_calendar_pcalendar (EPlugin *epl, EConfigHookItemFactoryData *data)
 	e_uri_free (uri);
 
 	parent = data->parent;
-	row = ((GtkTable*)parent)->nrows;
+	g_object_get (parent, "n-rows", &row, NULL);
 
 	exchange_config_listener_get_offline_status (exchange_global_config_listener,
 								    &offline_status);
@@ -527,7 +527,7 @@ e_exchange_calendar_commit (EPlugin *epl, EConfigTarget *target)
 	}
 
 	if (err_msg)
-		e_error_run (GTK_WINDOW (target->widget), err_msg, NULL);
+		e_alert_run_dialog_for_args (GTK_WINDOW (target->widget), err_msg, NULL);
  done:
 	g_free (uri_text);
 	g_free (username);
