@@ -32,6 +32,7 @@
 #include <e-util/e-alert-dialog.h>
 
 #include "exchange-send-options.h"
+#include "gtk-compat.h"
 
 G_DEFINE_TYPE (ExchangeSendOptionsDialog, exchange_sendoptions_dialog, G_TYPE_OBJECT)
 
@@ -109,7 +110,7 @@ exchange_send_options_get_widgets_data (ExchangeSendOptionsDialog *sod)
 
 		/* The temporary variables address, email, and name are needed to fetch the list items.
 		   Only the valid one is then copied into the storage variables. The "count" variable
-		   helps us keep a count of the exact number of items in the list. The g_list_length(GList *)
+		   helps us keep a count of the exact number of items in the list. The g_list_length (GList *)
 		   produced ambiguous results. Hence count is used :)
 		*/
 		for (; tmp != NULL; tmp = g_list_next (tmp)) {
@@ -385,24 +386,24 @@ exchange_sendoptions_dialog_run (ExchangeSendOptionsDialog *sod, GtkWidget *pare
 			  (GtkAttachOptions) (0), 0, 0);
 	gtk_misc_set_alignment (GTK_MISC (sensitivity_label), 0, 0.5);
 
-	sensitivity_combo_box = gtk_combo_box_new_text ();
+	sensitivity_combo_box = gtk_combo_box_text_new ();
 	gtk_widget_show (sensitivity_combo_box);
 	gtk_table_attach (GTK_TABLE (msg_settings_table), sensitivity_combo_box, 1, 2, 1, 2,
 			  (GtkAttachOptions) (GTK_FILL),
 			  (GtkAttachOptions) (GTK_FILL), 0, 0);
-	gtk_combo_box_append_text (GTK_COMBO_BOX (sensitivity_combo_box), _("Normal"));
-	gtk_combo_box_append_text (GTK_COMBO_BOX (sensitivity_combo_box), _("Personal"));
-	gtk_combo_box_append_text (GTK_COMBO_BOX (sensitivity_combo_box), _("Private"));
-	gtk_combo_box_append_text (GTK_COMBO_BOX (sensitivity_combo_box), _("Confidential"));
+	gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (sensitivity_combo_box), _("Normal"));
+	gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (sensitivity_combo_box), _("Personal"));
+	gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (sensitivity_combo_box), _("Private"));
+	gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (sensitivity_combo_box), _("Confidential"));
 
-	imp_combo_box = gtk_combo_box_new_text ();
+	imp_combo_box = gtk_combo_box_text_new ();
 	gtk_widget_show (imp_combo_box);
 	gtk_table_attach (GTK_TABLE (msg_settings_table), imp_combo_box, 1, 2, 0, 1,
 			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
 			  (GtkAttachOptions) (GTK_FILL), 0, 0);
-	gtk_combo_box_append_text (GTK_COMBO_BOX (imp_combo_box), _("Normal"));
-	gtk_combo_box_append_text (GTK_COMBO_BOX (imp_combo_box), _("High"));
-	gtk_combo_box_append_text (GTK_COMBO_BOX (imp_combo_box), _("Low"));
+	gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (imp_combo_box), _("Normal"));
+	gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (imp_combo_box), _("High"));
+	gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (imp_combo_box), _("Low"));
 
 	del_enabled_check = gtk_check_button_new_with_mnemonic (_("Send as Delegate"));
 	gtk_widget_show (del_enabled_check);
@@ -484,9 +485,9 @@ exchange_sendoptions_dialog_run (ExchangeSendOptionsDialog *sod, GtkWidget *pare
 	g_signal_connect (name_selector_dialog, "response",
 				G_CALLBACK (addressbook_dialog_response), sod);
 	g_signal_connect (GTK_DIALOG (priv->main), "response",
-				G_CALLBACK(exchange_send_options_cb), sod);
+				G_CALLBACK (exchange_send_options_cb), sod);
 	g_signal_connect ((GtkCheckButton *) priv->delegate_enabled, "toggled",
-				G_CALLBACK(delegate_option_toggled), sod);
+				G_CALLBACK (delegate_option_toggled), sod);
 
 	name_selector_entry = e_name_selector_peek_section_entry (priv->proxy_name_selector,
 									"Add User");
